package workforecast

import org.springframework.dao.DataIntegrityViolationException

class ActivityController {

    static allowedMethods = [save: "POST", update: "POST", delete: "POST"]

    def index() {
        redirect(action: "list", params: params)
    }

    def list(Integer max) {
        params.max = Math.min(max ?: 10, 100)
        [activityInstanceList: Activity.list(params), activityInstanceTotal: Activity.count()]
    }

    def create() {
		def activityInstance = new Activity(params)				
		activityInstance.forecast = Forecast.get(params.forecastId)

        [activityInstance: activityInstance]
    }

    def save() {
        def activityInstance = new Activity(params)
		activityInstance.forecast = Forecast.get(params.forecastId)
				
        if (!activityInstance.save(flush: true)) {
            render(view: "create", model: [activityInstance: activityInstance])
            return
        }
		
		def it = activityInstance.forecast.activities.iterator()
		int totalWorkload = 0
		while (it.hasNext()) {
			totalWorkload += it.next().workloadEstimation
		}

		if (totalWorkload > 100) {
			flash.warning = message(code: 'Workload is over 100%')
		}
		
        flash.message = message(code: 'default.created.message', args: [message(code: 'activity.label', default: 'Activity'), activityInstance.id])
        redirect(controller: "Forecast", action: "show", id: activityInstance.forecast.id)
    }

    def show(Long id) {
        def activityInstance = Activity.get(id)
		activityInstance.forecast = Forecast.get(params.forecastId)
		
        if (!activityInstance) {
            flash.message = message(code: 'default.not.found.message', args: [message(code: 'activity.label', default: 'Activity'), id])
            redirect(action: "list")
            return
        }

        [activityInstance: activityInstance]
    }

    def edit(Long id) {
		
		System.out.println("YYYY")
		System.out.println(params)
		
        def activityInstance = Activity.get(id)
		activityInstance.forecast = Forecast.get(params.forecastId)
		
        if (!activityInstance) {
            flash.message = message(code: 'default.not.found.message', args: [message(code: 'activity.label', default: 'Activity'), id])
            redirect(action: "list")
            return
        }

        [activityInstance: activityInstance]
    }

    def update(Long id, Long version) {
        def activityInstance = Activity.get(id)
		activityInstance.forecast = Forecast.get(params.forecastId)
		
        if (!activityInstance) {
            flash.message = message(code: 'default.not.found.message', args: [message(code: 'activity.label', default: 'Activity'), id])
            redirect(action: "list")
            return
        }

        if (version != null) {
            if (activityInstance.version > version) {
                activityInstance.errors.rejectValue("version", "default.optimistic.locking.failure",
                          [message(code: 'activity.label', default: 'Activity')] as Object[],
                          "Another user has updated this Activity while you were editing")
                render(view: "edit", model: [activityInstance: activityInstance])
                return
            }
        }

        activityInstance.properties = params

        if (!activityInstance.save(flush: true)) {
            render(view: "edit", model: [activityInstance: activityInstance])
            return
        }
		
		def it = activityInstance.forecast.activities.iterator()
		int totalWorkload = 0
		while (it.hasNext()) {
			totalWorkload += it.next().workloadEstimation
		}

		if (totalWorkload > 100) {
			flash.warning = message(code: 'Workload is over 100%')
		}
		
        flash.message = message(code: 'default.updated.message', args: [message(code: 'activity.label', default: 'Activity'), activityInstance.id])
        redirect(controller: "Forecast", action: "show", id: activityInstance.forecast.id)
    }

    def delete(Long id) {
        def activityInstance = Activity.get(id)
		activityInstance.forecast = Forecast.get(params.forecastId)
		
        if (!activityInstance) {
            flash.message = message(code: 'default.not.found.message', args: [message(code: 'activity.label', default: 'Activity'), id])
            redirect(action: "list")
            return
        }

        try {
            activityInstance.delete(flush: true)
            flash.message = message(code: 'default.deleted.message', args: [message(code: 'activity.label', default: 'Activity'), id])
			redirect(controller: "Forecast", action: "show", id: activityInstance.forecast.id)
        }
        catch (DataIntegrityViolationException e) {
            flash.message = message(code: 'default.not.deleted.message', args: [message(code: 'activity.label', default: 'Activity'), id])
			redirect(controller: "Forecast", action: "show", id: activityInstance.forecast.id)
        }
    }
}
